from datetime import datetime

from django.contrib.auth.models import User
from django.db import models
from notifications.backend.django_email import DjangoEmailNotificationBackend
from notifications.engine import NotificationEngine

from notifications.fields import JSONField
from notifications.type.account import AccountNotification
from notifications.type.default import DefaultNotification


class NotificationQueue(models.Model):
    """
    The queue is used for backends that have show_method set to "queue"
    """
    user = models.ForeignKey(User, related_name='notifications')
    subject = models.CharField(max_length=255, blank=True)
    text = models.TextField(blank=True)
    tries = models.PositiveIntegerField(default=0)
    send_at = models.DateTimeField(blank=True, null=True)
    notification_backend = models.CharField(max_length=255)

    def get_backend(self):
        return NotificationEngine._backends[self.notification_backend]

    def __unicode__(self):
        return self.text


class DisabledNotificationsTypeBackend(models.Model):
    """
    In this model we save which backends a user does NOT want to use for a notification type.
    By saving what he does NOT want, new users and new types/backends default to everything.
    """
    user = models.ForeignKey(User)
    notification_type = models.CharField(max_length=255)
    notification_backends = models.TextField()

    def get_backends(self):
        return self.notification_backends.split(',')

    class Meta:
        unique_together = ['user', 'notification_type']


class NotificationBackendSettings(models.Model):
    """
    Model for saving required settings per backend
    """
    user = models.ForeignKey(User)
    notification_backend = models.CharField(max_length=255)
    settings = JSONField()

    class Meta:
        unique_together = ['user', 'notification_backend']



from django.conf.urls.defaults import patterns, url
from django.contrib.auth.decorators import login_required
from notifications.views import IndexView


urlpatterns = patterns('',
    url(r'^$', login_required(IndexView.as_view()), name='notification-settings-index'),
)



class AlreadyRegistered(Exception):
    pass


class NotRegistered(Exception):
    pass


class NotificationEngine(object):
    _types = {}
    _backends = {}

    @classmethod
    def register_backend(cls, backend):
        key = backend.__name__

        if key in cls._backends:
            raise AlreadyRegistered('The notification backend %s is already registered' % key)

        cls._backends[key] = backend

    @classmethod
    def unregister_backend(cls, backend):
        key = backend.__name__

        if key not in cls._backends:
            raise NotRegistered('The notification backend %s is not registered' % key)

        del cls._backends[key]

    @classmethod
    def register_type(cls, type):
        key = type.__name__

        if key in cls._types:
            raise AlreadyRegistered('The notification type %s is already registered' % key)

        cls._types[key] = type

    @classmethod
    def unregister_type(cls, type):
        key = type.__name__

        if key not in cls._types:
            raise NotRegistered('The notification type %s is not registered' % key)

        del cls._types[key]



from distutils.core import setup
from notifications import __version__ as version

setup(
    name="django-generic-notifications",
    version=version,
    description="Generic notification system for Django, with multiple input types and output backends",
    long_description=open("README.rst").read(),
    author="Kevin Renskers",
    author_email="info@mixedcase.nl",
    url="https://github.com/kevinrenskers/django-generic-notifications",
    packages=[
        "notifications",
    ],
    install_requires=[
        "django >= 1.2",
    ],
    package_dir={"notifications": "notifications"},
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.5',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        "Framework :: Django",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ]
)



import datetime
from decimal import Decimal
from django.db import models
from django.conf import settings
from django.utils import simplejson


class JSONEncoder(simplejson.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, Decimal):
            return str(obj)
        elif isinstance(obj, datetime.datetime):
            assert settings.TIME_ZONE == 'UTC'
            return obj.strftime('%Y-%m-%dT%H:%M:%SZ')
        return simplejson.JSONEncoder.default(self, obj)


def dumps(value):
    assert isinstance(value, dict)
    return JSONEncoder().encode(value)


def loads(txt):
    value = simplejson.loads(
        txt,
        parse_float=Decimal,
        encoding=settings.DEFAULT_CHARSET
    )
    assert isinstance(value, dict)
    return value


class JSONDict(dict):
    """
    Hack so repr() called by dumpdata will output JSON instead of
    Python formatted data.  This way fixtures will work!
    """
    def __repr__(self):
        return dumps(self)


class JSONField(models.TextField):
    """JSONField is a generic textfield that neatly serializes/unserializes
    JSON objects seamlessly.  Main thingy must be a dict object."""

    # Used so to_python() is called
    __metaclass__ = models.SubfieldBase

    def __init__(self, *args, **kwargs):
        if 'default' not in kwargs:
            kwargs['default'] = '{}'
        models.TextField.__init__(self, *args, **kwargs)

    def to_python(self, value):
        """Convert our string value to JSON after we load it from the DB"""
        if not value:
            return {}
        elif isinstance(value, basestring):
            res = loads(value)
            assert isinstance(res, dict)
            return JSONDict(**res)
        else:
            return value

    def get_db_prep_save(self, value, connection):
        """Convert our JSON object to a string before we save"""
        if not value:
            return super(JSONField, self).get_db_prep_save("")
        else:
            return super(JSONField, self).get_db_prep_save(dumps(value))

    def south_field_triple(self):
        "Returns a suitable description of this field for South."
        # We'll just introspect the _actual_ field.
        from south.modelsinspector import introspector
        field_class = "django.db.models.fields.TextField"
        args, kwargs = introspector(self)
        # That's our definition!
        return (field_class, args, kwargs)



#
# Convince South to to migrate fields of this type
#
try:
    from south.modelsinspector import add_introspection_rules
    add_introspection_rules([], ["^notifications\.fields\.JSONField"])
except:
    pass



from django.contrib import messages
from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect
from django.views.generic.base import TemplateView
from notifications.engine import NotificationEngine
from notifications.models import DisabledNotificationsTypeBackend


class IndexView(TemplateView):
    template_name = 'notifications/index.html'

    def get_context_data(self, **kwargs):
        disabled_backends = {}

        for obj in DisabledNotificationsTypeBackend.objects.filter(user=self.request.user):
            disabled_backends[obj.notification_type] = obj.get_backends()

        notification_types = {}
        for class_name, type_class in NotificationEngine._types.items():
            notification_types[class_name] = {
                'type_class': type_class,
                'disabled_backends': disabled_backends.get(class_name, [])
            }

        context = super(IndexView, self).get_context_data(**kwargs)
        context['notification_types'] = notification_types
        context['notification_backends'] = NotificationEngine._backends
        return context

    def post(self, request, **kwargs):
        DisabledNotificationsTypeBackend.objects.filter(user=request.user).delete()

        for type_name, type_class in NotificationEngine._types.items():
            allowed_backends = type_class().allowed_backends
            enabled_backends = request.POST.getlist(type_name)

            # Subtract the enabled backends from allowed_backends, the difference is disabled_backends
            disabled_backends = list(allowed_backends)
            [disabled_backends.remove(x) for x in enabled_backends]

            if disabled_backends:
                DisabledNotificationsTypeBackend.objects.create(
                    user = request.user,
                    notification_type = type_name,
                    notification_backends = ','.join(disabled_backends)
                )

        messages.success(request, 'Your notification settings have been saved')
        return HttpResponseRedirect(reverse('notification-settings-index'))
